program crc1632;
uses
  crc;

{$IFDEF WIN32}
{$APPTYPE CONSOLE}
{$ENDIF}
{$I-}

{$I ..\LIB\VERSION.INC}

const
  crc1632version = 'crc1632 '+BT_VERSION+' for ' + {$IFDEF WIN32} 'win32'{$ELSE}
  'dos'{$ENDIF};
  cleft =
    '(C) markus stephany, own parts released under the terms of the GNU GPL';

  (* /? parameter *)

procedure Usage;
begin
  writeln(crc1632version);
  writeln(cleft);
  writeln;
  writeln('crc1632 calculates and prints the crc16 or crc32 of either the contents of a');
  writeln('file or a text from STDIN.');
  writeln;
  writeln('usage:');
  writeln('crc1632 [/m<method>] [/f<file name>] [/v<variable name>] [/d]');
  writeln;
  writeln('  /m<method>:');
  writeln('    either /m32 or /m16. if omitted, the crc32 checksum will be calculated.');
  writeln;
  writeln('  /f<file name>:');
  writeln('    if this parameter is specified, crc1632 will calculate the checksum of');
  writeln('    the contents of the specified file. if no filename parameter is given');
  writeln('    crc1632 will read the data from STDIN.');
  writeln;
  writeln('  /v<variable name>:');
  writeln('    if a variable name is specified on the command line, the output will be');
  writeln('    modified to a SET command (SET <variable name>=<crc>)');
  writeln;
  writeln('  /d:');
  writeln('    if the /d flag is specified, the checksum will be printed in decimal');
  writeln('    format, otherwise it will be printed in hexadecimal format.');
  writeln;
  writeln('examples:');
  writeln('  crc1632 /m16 /ffile.ext /vCRC /d');
  writeln('    will calculate the 16 bit checksum of the file "file.ext" and print a');
  writeln('    "SET" command to STDOUT, crc in decimal notation (e.g.');
  writeln('    "SET CRC=14217")');
  writeln;
  writeln('  crc1632');
  writeln('    will calculate the 32 bit checksum of the data read from STDIN and');
  writeln('    print the checksum in hexadecimal notation (e.g. "002AB5D6")');
  writeln;
  Halt(1);
end;

function ByteToHex(b: byte): string;
const
  HEX: array[0..15] of char = '0123456789ABCDEF';
begin
  ByteToHex := HEX[b shr 4] + HEX[b and 15];
end;

var
  BDo16: Boolean; (* /m16 flag => crc16 *)
  SFile: string; (* filename in /f *)
  SVar: string; (* variable name in /v *)
  BDecimal: Boolean; (* /d flag => decimal *)

var
  i: Integer;
  c: Char;
  s: string;

  cs: LongInt;
begin
  FileMode := 0;
  BDo16 := False;
  SFile := '';
  SVar := '';
  BDecimal := False;
  if ParamStr(1) = '/?' then
    Usage;

  {Check command line}
  if ParamCount > 0 then
    for i := 1 to ParamCount do
    begin
      s := Copy(ParamStr(i), 2, 1) + #0;
      c := s[1];
      case UpCase(c) of
        'V':
          begin
            SVar := Copy(ParamStr(i), 3, MaxInt);
          end;
        'D':
          begin
            BDecimal := True;
          end;
        'H':
          begin
            BDecimal := False;
          end;
        'M':
          begin
            s := Copy(ParamStr(i), 3, MaxInt);
            if s = '16' then
              BDo16 := True
            else if s = '32' then
              BDo16 := False
            else
            begin
              writeln('Unknown parameter: ', ParamStr(i));
              Halt(1);
            end;
          end;
        'F':
          begin
            SFile := Copy(ParamStr(i), 3, MaxInt);
          end;
      else
        writeln('Unknown parameter: ', ParamStr(i));
        Halt(1);
      end;
    end;

  if BDo16 then
    cs := FileCRC16(SFile)
  else
    cs := FileCRC32(SFile);

  if BDecimal then
    Str(cs, S)
  else
  begin
    if BDo16 then
      // error until 09-27-2004: was ByteToHex(cs and 15)!!!
      S := ByteToHex((cs shr 8) and 255) + ByteToHex(cs and 255)
    else
      S := ByteToHex((cs shr 24) and 255) + ByteToHex((cs shr 16) and 255) +
        ByteToHex((cs shr 8) and 255) + ByteToHex(cs and 255)
  end;
  if SVar <> '' then
    writeln('SET ', SVar, '=', S)
  else
    writeln(S);
end.

