program forcedir;

(*

  creates the whole specified directory path
  (C) markus stephany, released under the terms of the GNU GPL version 2.

*)

{$APPTYPE CONSOLE}
uses
  SysConst,
  SysUtils;

{$I ..\LIB\VERSION.INC}

procedure ForceDirs(Dir: string);
var
  E: EInOutError;
begin
  if Dir = '' then
  begin
    E := EInOutError.Create(SCannotCreateDir);
    E.ErrorCode := 3;
    raise E;
  end;
  Dir := ExcludeTrailingPathDelimiter(Dir);
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit;
  ForceDirs(ExtractFilePath(Dir));
  if not CreateDir(Dir) then
    RaiseLastOSError;
end;

var
  dir: string;

begin
  if (ParamStr(1) = '/?') or (ParamCount <> 1) then
  begin
    writeln('forcedir creates the whole directory path whose name is given');
    writeln('on the command line. an exit code of 1 is returned if the');
    writeln('specified directory cannot be created');
    writeln('(C) markus stephany, released under the terms of the GNU GPL.');
    writeln('forcedir version: '+BT_VERSION);
    Halt(1);
  end;
  try
    dir := ExpandFileName(ParamStr(1));
    ForceDirs(dir);
    if not DirectoryExists(dir) then
      RaiseLastOSError;
  except
    on E: Exception do
    begin
      writeln(ErrOutput, 'forcedir: cannot create directory ', dir, ': ',
        E.Message);
      Halt(1);
    end;
  end;
  Halt(0);
end.

