program gpcvreg;

{$APPTYPE CONSOLE}

uses
  Windows,
  SysUtils,
  Classes,
  utils,
  cl_utils;

resourcestring
  ERR_UNKNOWN_DATA = 'unknown data in line %d: %s';
  ERR_NO_KEY = 'no key name was given in line %d';
  ERR_UNKNOWN_FORMAT = 'unknown data format in line %d: %s';

{$I ..\LIB\VERSION.INC}
const
  GPCVRegVersion = 'gpcvreg '+BT_VERSION; // my own version


  // show help

procedure Usage;
begin
  writeln(ErrOutput, GPCVRegVersion);
  writeln(ErrOutput, CLEFT);
  writeln(ErrOutput);
  writeln(ErrOutput,
    'gpcvreg converts .reg files to policy rules readable by gpcscript.');
  writeln(ErrOutput);
  writeln(ErrOutput, 'usage:');
  writeln(ErrOutput, '<pipe in>|gpcvreg|<pipe out>');
  writeln(ErrOutput);
  writeln(ErrOutput, 'example:');
  writeln(ErrOutput, 'type user.reg|gpcvreg|gpscript user /file');
  Halt(1);
end;

var
  slIn: TStrings;
  sTemp: string = '';
  sKey: string = '';
  sValue: string = '';
  sData: string = '';
  dwType: Cardinal;
  intLoop: Integer = 1;
  intPos,
    intData: Integer;
  bFirst: Boolean = False;

begin
  if FindACmdLineSwitch(['version']) then
  begin
    writeln(GPCVRegVersion);
    Halt(0);
  end;

  if FindACmdLineSwitch(['h', 'help', '?', 'usage']) then
    Usage;

  try
    slIn := TStringList.Create;
    try
      GetStdInList(slIn);
      with slIn do
      begin
        if Bool(Count) then
          while Count > 0 do
          begin
            sTemp := Trim(Strings[0]);
            if ((sTemp <> '') and (sTemp[1] in ['#', ';'])) then
            begin
              system.Delete(sTemp, 1, 1);
              sTemp := Trim(sTemp);
              writeln('# ', sTemp);
              Delete(0);
              inc(intLoop);
            end
            else if ((Copy(sTemp, 1, 1) <> '[') and (not bFirst)) or (sTemp = '')
              then
            begin
              Delete(0);
              inc(intLoop);
            end
            else
            begin
              bFirst := True;
              if (sTemp[1] = '[') and (sTemp[Length(sTemp)] = ']') then
              begin
                // key
                sKey := Copy(sTemp, 2, Length(sTemp) - 2);
                // test for HKEY_ prefix
                if Pos('HKEY_', UpperCase(sKey)) = 1 then
                begin
                  intPos := Pos('\', sKey);
                  if intPos = 0 then
                    sKey := ''
                  else
                    sKey := Copy(sKey, intPos + 1, MaxInt);
                end
                else if sKey[1] = '-' then
                begin
                  // deletekey not supported
                  writeln(ErrOutput,'WARNING: deletekey not supported in line ',
                    intLoop);
                  ExitCode := 2;
                  sKey := '';
                end;
                Delete(0);
                inc(intLoop);
              end
              else
              begin
                // must be a value @|"val"=[type:]data
                if not (sTemp[1] in ['"', '@']) then
                  raise Exception.CreateFmt(ERR_UNKNOWN_DATA, [intLoop, sTemp]);
                // a [key] must have been specified previously
                if sKey = '' then
                  raise Exception.CreateFmt(ERR_NO_KEY, [intLoop, sTemp]);
                intPos := Pos('=', sTemp);
                if intPos = 0 then
                  raise Exception.CreateFmt(ERR_UNKNOWN_FORMAT, [intLoop,
                    sTemp]);
                sValue := Trim(Copy(sTemp, 1, intPos - 1));
                if sValue = '@' then
                  sValue := ''
                else
                  sValue := UnQuote(sValue);
                sTemp := Trim(Copy(sTemp, intPos + 1, MaxInt));
                if sTemp = '-' then
                begin
                  // delete data
                  Delete(0);
                  inc(intLoop);
                  writeln('/KEY:"', sKey, '"');
                  writeln('/VALUE:"', sValue, '"');
                  writeln('/DELETE');
                  writeln;
                end
                else
                begin
                  if sTemp = '' then
                    raise Exception.CreateFmt(ERR_UNKNOWN_FORMAT, [intLoop,
                      sTemp]);

                  if sTemp[1] = '"' then
                  begin
                    // string
                    dwType := REG_SZ;
                    sTemp := UnQuote(sTemp);
                    sData := 'S "' + sTemp + '"';
                  end
                  else
                  begin
                    if Pos('dword:', LowerCase(sTemp)) = 1 then
                    begin
                      // dword
                      dwType := REG_DWORD;
                      sTemp := Trim(Copy(sTemp, 7, MaxInt));
                      if not TryStrToInt('0x' + sTemp, intData) then
                        raise Exception.CreateFmt(ERR_UNKNOWN_FORMAT, [intLoop,
                          sTemp]);
                      sData := GetDWORDValue(sizeof(intData), @intData);
                    end
                    else
                    begin
                      if Pos('hex', LowerCase(sTemp)) <> 1 then
                        raise Exception.CreateFmt(ERR_UNKNOWN_FORMAT, [intLoop,
                          sTemp]);
                      // binary
                      dwType := REG_BINARY;
                      sTemp := Copy(sTemp, 4, MaxInt);
                      intPos := Pos(':', sTemp);
                      if intPos = 0 then
                        raise Exception.CreateFmt(ERR_UNKNOWN_FORMAT, [intLoop,
                          sTemp]);
                      sData := 'B ' + Trim(Copy(sTemp, intPos + 1, MaxInt));
                      sTemp := Copy(sTemp, 1, intPos - 1);
                      if sTemp <> '' then
                      begin
                        if (sTemp[1] <> '(') or (sTemp[Length(sTemp)] <> ')') or
                        (not TryStrToInt('0x' + Copy(sTemp, 2, Length(sTemp) -
                          2), intData)) then
                          raise Exception.CreateFmt(ERR_UNKNOWN_FORMAT,
                            [intLoop, sTemp]);
                        dwType := intData;
                      end;
                      // now look for multiline values
                      sData := Trim(sData);
                      while (Copy(sData, Length(sData), 1) = '\') and
                        (Bool(Count)) do
                      begin
                        Delete(0);
                        inc(intLoop);
                        sData := sData + #13#10 + Strings[0];
                      end;
                    end;
                  end;
                  Delete(0);
                  inc(intLoop);
                  writeln('/KEY:"', sKey, '"');
                  writeln('/VALUE:"', sValue, '"');
                  writeln('/TYPE:', GetDataTypeString(dwType));
                  writeln('/DATA:', sData);
                  writeln('/SET');
                  writeln;
                end;
              end;
            end;
          end;

      end;
    finally
      slIn.Free;
    end;
  except
    on E: Exception do
    begin
      WriteLn(ErrOutput, ToOem(Format(STR_ERROR, [E.Message])));
      if ExitCode = 0 then
        ExitCode := 1;
    end;
  end;
end.

