program iskeydown;

(*

  checks if the key(s) given on the command line are currently pressed
  and returns 0 if all of them are down, 1 if not all of them are
  down, 2 if an error occurred.
  you can either use the VK_XYZ windows api key constants or the
  numerical scancode values.
  (C) markus stephany, released under the terms of the GNU GPL.

*)

{$APPTYPE CONSOLE}
uses
  Windows;

{$I ..\LIB\VERSION.INC}

type
  TKeyRec = record
    N: string;
    C: integer;
    D: string;
  end;

const
  KEY_NAMES: array[0..172] of TKeyRec = (
    (N: 'VK_LBUTTON';    C: $01; D:'Left mouse button'),
    (N: 'VK_RBUTTON';    C: $02; D:'Right mouse button'),
    (N: 'VK_CANCEL';     C: $03; D:'Control-break processing'),
    (N: 'VK_MBUTTON';    C: $04; D:'Middle mouse button (three-button mouse)'),
    (N: 'VK_XBUTTON1';   C: $05; D:'Windows 2000/XP: X1 mouse button'),
    (N: 'VK_XBUTTON2';   C: $06; D:'Windows 2000/XP: X2 mouse button'),
    (N: 'VK_BACK';       C: $08; D:'BACKSPACE key'),
    (N: 'VK_TAB';        C: $09; D:'TAB key'),
    (N: 'VK_CLEAR';      C: $0C; D:'CLEAR key'),
    (N: 'VK_RETURN';     C: $0D; D:'ENTER key'),
    (N: 'VK_SHIFT';      C: $10; D:'SHIFT key'),
    (N: 'VK_CONTROL';    C: $11; D:'CTRL key'),
    (N: 'VK_MENU';       C: $12; D:'ALT key'),
    (N: 'VK_PAUSE';      C: $13; D:'PAUSE key'),
    (N: 'VK_CAPITAL';    C: $14; D:'CAPS LOCK key'),
    (N: 'VK_KANA';       C: $15; D:'Input Method Editor (IME) Kana mode'),
    (N: 'VK_HANGUEL';    C: $15; D:'IME Hanguel mode (maintained for compatibility; use VK_HANGUL)'),
    (N: 'VK_HANGUL';     C: $15; D:'IME Hangul mode'),
    (N: 'VK_JUNJA';      C: $17; D:'IME Junja mode'),
    (N: 'VK_FINAL';      C: $18; D:'IME final mode'),
    (N: 'VK_HANJA';      C: $19; D:'IME Hanja mode'),
    (N: 'VK_KANJI';      C: $19; D:'IME Kanji mode'),
    (N: 'VK_ESCAPE';     C: $1B; D:'ESC key'),
    (N: 'VK_CONVERT';    C: $1C; D:'IME convert'),
    (N: 'VK_NONCONVERT'; C: $1D; D:'IME nonconvert'),
    (N: 'VK_ACCEPT';     C: $1E; D:'IME accept'),
    (N: 'VK_MODECHANGE'; C: $1F; D:'IME mode change request'),
    (N: 'VK_SPACE';      C: $20; D:'SPACEBAR'),
    (N: 'VK_PRIOR';      C: $21; D:'PAGE UP key'),
    (N: 'VK_NEXT';       C: $22; D:'PAGE DOWN key'),
    (N: 'VK_END';        C: $23; D:'END key'),
    (N: 'VK_HOME';       C: $24; D:'HOME key'),
    (N: 'VK_LEFT';       C: $25; D:'LEFT ARROW key'),
    (N: 'VK_UP';         C: $26; D:'UP ARROW key'),
    (N: 'VK_RIGHT';      C: $27; D:'RIGHT ARROW key'),
    (N: 'VK_DOWN';       C: $28; D:'DOWN ARROW key'),
    (N: 'VK_SELECT';     C: $29; D:'SELECT key'),
    (N: 'VK_PRINT';      C: $2A; D:'PRINT key'),
    (N: 'VK_EXECUTE';    C: $2B; D:'EXECUTE key'),
    (N: 'VK_SNAPSHOT';   C: $2C; D:'PRINT SCREEN key'),
    (N: 'VK_INSERT';     C: $2D; D:'INS key'),
    (N: 'VK_DELETE';     C: $2E; D:'DEL key'),
    (N: 'VK_HELP';       C: $2F; D:'HELP key'),
    (N: 'VK_0';          C: $30; D:'0 key'),
    (N: 'VK_1';          C: $31; D:'1 key'),
    (N: 'VK_2';          C: $32; D:'2 key'),
    (N: 'VK_3';          C: $33; D:'3 key'),
    (N: 'VK_4';          C: $34; D:'4 key'),
    (N: 'VK_5';          C: $35; D:'5 key'),
    (N: 'VK_6';          C: $36; D:'6 key'),
    (N: 'VK_7';          C: $37; D:'7 key'),
    (N: 'VK_8';          C: $38; D:'8 key'),
    (N: 'VK_9';          C: $39; D:'9 key'),
    (N: 'VK_A';          C: $41; D:'A key'),
    (N: 'VK_B';              C: $42; D:'B key'),
    (N: 'VK_C';              C: $43; D:'C key'),
    (N: 'VK_D';              C: $44; D:'D key'),
    (N: 'VK_E';              C: $45; D:'E key'),
    (N: 'VK_F';              C: $46; D:'F key'),
    (N: 'VK_G';              C: $47; D:'G key'),
    (N: 'VK_H';              C: $48; D:'H key'),
    (N: 'VK_I';              C: $49; D:'I key'),
    (N: 'VK_J';              C: $4A; D:'J key'),
    (N: 'VK_K';              C: $4B; D:'K key'),
    (N: 'VK_L';              C: $4C; D:'L key'),
    (N: 'VK_M';              C: $4D; D:'M key'),
    (N: 'VK_N';              C: $4E; D:'N key'),
    (N: 'VK_O';              C: $4F; D:'O key'),
    (N: 'VK_P';              C: $50; D:'P key'),
    (N: 'VK_Q';              C: $51; D:'Q key'),
    (N: 'VK_R';              C: $52; D:'R key'),
    (N: 'VK_S';              C: $53; D:'S key'),
    (N: 'VK_T';              C: $54; D:'T key'),
    (N: 'VK_U';              C: $55; D:'U key'),
    (N: 'VK_V';              C: $56; D:'V key'),
    (N: 'VK_W';              C: $57; D:'W key'),
    (N: 'VK_X';              C: $58; D:'X key'),
    (N: 'VK_Y';              C: $59; D:'Y key'),
    (N: 'VK_Z';              C: $5A; D:'Z key'),
    (N: 'VK_LWIN';       C: $5B; D:'Left Windows key'),
    (N: 'VK_RWIN';       C: $5C; D:'Right Windows key'),
    (N: 'VK_APPS';       C: $5D; D:'Applications key'),
    (N: 'VK_SLEEP';      C: $5F; D:'Computer Sleep key'),
    (N: 'VK_NUMPAD0';    C: $60; D:'Numeric keypad 0 key'),
    (N: 'VK_NUMPAD1';    C: $61; D:'Numeric keypad 1 key'),
    (N: 'VK_NUMPAD2';    C: $62; D:'Numeric keypad 2 key'),
    (N: 'VK_NUMPAD3';    C: $63; D:'Numeric keypad 3 key'),
    (N: 'VK_NUMPAD4';    C: $64; D:'Numeric keypad 4 key'),
    (N: 'VK_NUMPAD5';    C: $65; D:'Numeric keypad 5 key'),
    (N: 'VK_NUMPAD6';    C: $66; D:'Numeric keypad 6 key'),
    (N: 'VK_NUMPAD7';    C: $67; D:'Numeric keypad 7 key'),
    (N: 'VK_NUMPAD8';    C: $68; D:'Numeric keypad 8 key'),
    (N: 'VK_NUMPAD9';    C: $69; D:'Numeric keypad 9 key'),
    (N: 'VK_MULTIPLY';   C: $6A; D:'Multiply key'),
    (N: 'VK_ADD';        C: $6B; D:'Add key'),
    (N: 'VK_SEPARATOR';  C: $6C; D:'Separator key'),
    (N: 'VK_SUBTRACT';   C: $6D; D:'Subtract key'),
    (N: 'VK_DECIMAL';    C: $6E; D:'Decimal key'),
    (N: 'VK_DIVIDE';     C: $6F; D:'Divide key'),
    (N: 'VK_F1';         C: $70; D:'F1 key'),
    (N: 'VK_F2';         C: $71; D:'F2 key'),
    (N: 'VK_F3';         C: $72; D:'F3 key'),
    (N: 'VK_F4';         C: $73; D:'F4 key'),
    (N: 'VK_F5';         C: $74; D:'F5 key'),
    (N: 'VK_F6';         C: $75; D:'F6 key'),
    (N: 'VK_F7';         C: $76; D:'F7 key'),
    (N: 'VK_F8';         C: $77; D:'F8 key'),
    (N: 'VK_F9';         C: $78; D:'F9 key'),
    (N: 'VK_F10';        C: $79; D:'F10 key'),
    (N: 'VK_F11';        C: $7A; D:'F11 key'),
    (N: 'VK_F12';        C: $7B; D:'F12 key'),
    (N: 'VK_F13';        C: $7C; D:'F13 key'),
    (N: 'VK_F14';        C: $7D; D:'F14 key'),
    (N: 'VK_F15';        C: $7E; D:'F15 key'),
    (N: 'VK_F16';        C: $7F; D:'F16 key'),
    (N: 'VK_F17';        C: $80; D:'F17 key'),
    (N: 'VK_F18';        C: $81; D:'F18 key'),
    (N: 'VK_F19';        C: $82; D:'F19 key'),
    (N: 'VK_F20';        C: $83; D:'F20 key'),
    (N: 'VK_F21';        C: $84; D:'F21 key'),
    (N: 'VK_F22';        C: $85; D:'F22 key'),
    (N: 'VK_F23';        C: $86; D:'F23 key'),
    (N: 'VK_F24';        C: $87; D:'F24 key'),
    (N: 'VK_NUMLOCK';    C: $90; D:'NUM LOCK key'),
    (N: 'VK_SCROLL';     C: $91; D:'SCROLL LOCK key'),
    (N: 'VK_LSHIFT';     C: $A0; D:'Left SHIFT key'),
    (N: 'VK_RSHIFT';     C: $A1; D:'Right SHIFT key'),
    (N: 'VK_LCONTROL';   C: $A2; D:'Left CONTROL key'),
    (N: 'VK_RCONTROL';   C: $A3; D:'Right CONTROL key'),
    (N: 'VK_LMENU';      C: $A4; D:'Left MENU key'),
    (N: 'VK_RMENU';      C: $A5; D:'Right MENU key'),
    (N: 'VK_BROWSER_BACK';  C: $A6; D:'Windows 2000/XP: Browser Back key'),
    (N: 'VK_BROWSER_FORWARD';  C: $A7; D:'Windows 2000/XP: Browser Forward key'),
    (N: 'VK_BROWSER_REFRESH';  C: $A8; D:'Windows 2000/XP: Browser Refresh key'),
    (N: 'VK_BROWSER_STOP';     C: $A9; D:'Windows 2000/XP: Browser Stop key'),
    (N: 'VK_BROWSER_SEARCH';   C: $AA; D:'Windows 2000/XP: Browser Search key'),
    (N: 'VK_BROWSER_FAVORITES';   C: $AB; D:'Windows 2000/XP: Browser Favorites key'),
    (N: 'VK_BROWSER_HOME';        C: $AC; D:'Windows 2000/XP: Browser Start and Home key'),
    (N: 'VK_VOLUME_MUTE';         C: $AD; D:'Windows 2000/XP: Volume Mute key'),
    (N: 'VK_VOLUME_DOWN';         C: $AE; D:'Windows 2000/XP: Volume Down key'),
    (N: 'VK_VOLUME_UP';           C: $AF; D:'Windows 2000/XP: Volume Up key'),
    (N: 'VK_MEDIA_NEXT_TRACK';    C: $B0; D:'Windows 2000/XP: Next Track key'),
    (N: 'VK_MEDIA_PREV_TRACK';    C: $B1; D:'Windows 2000/XP: Previous Track key'),
    (N: 'VK_MEDIA_STOP';          C: $B2; D:'Windows 2000/XP: Stop Media key'),
    (N: 'VK_MEDIA_PLAY_PAUSE';    C: $B3; D:'Windows 2000/XP: Play/Pause Media key'),
    (N: 'VK_LAUNCH_MAIL';         C: $B4; D:'Windows 2000/XP: Start Mail key'),
    (N: 'VK_LAUNCH_MEDIA_SELECT'; C: $B5; D:'Windows 2000/XP: Select Media key'),
    (N: 'VK_LAUNCH_APP1';         C: $B6; D:'Windows 2000/XP: Start Application 1 key'),
    (N: 'VK_LAUNCH_APP2';         C: $B7; D:'Windows 2000/XP: Start Application 2 key'),
    (N: 'VK_OEM_1';               C: $BA; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the ";:" key'),
    (N: 'VK_OEM_PLUS';            C: $BB; D:'Windows 2000/XP: For any country/region, the "+" key'),
    (N: 'VK_OEM_COMMA';           C: $BC; D:'Windows 2000/XP: For any country/region, the "," key'),
    (N: 'VK_OEM_MINUS';           C: $BD; D:'Windows 2000/XP: For any country/region, the "-" key'),
    (N: 'VK_OEM_PERIOD';          C: $BE; D:'Windows 2000/XP: For any country/region, the "." key'),
    (N: 'VK_OEM_2';               C: $BF; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the "/?" key'),
    (N: 'VK_OEM_3';               C: $C0; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the "`~" key'),
    (N: 'VK_OEM_4';               C: $DB; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the "[{" key'),
    (N: 'VK_OEM_5';               C: $DC; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the "\|" key'),
    (N: 'VK_OEM_6';               C: $DD; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the "]}" key'),
    (N: 'VK_OEM_7';               C: $DE; D:'Used for miscellaneous characters; it can vary by keyboard. Windows 2000/XP: For the US standard keyboard, the "single-quote/double-quote" key'),
    (N: 'VK_OEM_8';               C: $DF; D:'Used for miscellaneous characters; it can vary by keyboard.'),
    (N: 'VK_OEM_102';             C: $E2; D:'Windows 2000/XP: Either the angle bracket key or the backslash key on the RT 102-key keyboard'),
    (N: 'VK_PROCESSKEY';          C: $E5; D:'Windows 95/98/Me, Windows NT 4.0, Windows 2000/XP: IME PROCESS key'),
    (N: 'VK_PACKET';              C: $E7; D:'Windows 2000/XP: Used to pass Unicode characters as if they were keystrokes'),
    (N: 'VK_ATTN';                C: $F6; D:'Attn key'),
    (N: 'VK_CRSEL';               C: $F7; D:'CrSel key'),
    (N: 'VK_EXSEL';               C: $F8; D:'ExSel key'),
    (N: 'VK_EREOF';               C: $F9; D:'Erase EOF key'),
    (N: 'VK_PLAY';                C: $FA; D:'Play key'),
    (N: 'VK_ZOOM';                C: $FB; D:'Zoom key'),
    (N: 'VK_NONAME';              C: $FC; D:'Reserved for future use'),
    (N: 'VK_PA1';                 C: $FD; D:'PA1 key'),
    (N: 'VK_OEM_CLEAR';           C: $FE; D:'Clear key'));


function GetKey(s: string): integer;
var
  i: integer;
begin
  Result := -1;
  if s <> '' then
  begin
    Val(s,Result, i);
    if i <> 0 then
    begin
      Result := -1;
      for i := 1 to Length(s) do
        s[i] := UpCase(s[i]);
      for i := Low(KEY_NAMES) to High(KEY_NAMES) do
        if KEY_NAMES[i].N = s then
        begin
          Result := KEY_NAMES[i].C;
          Break;
        end;
    end;
  end;
end;

var
  i,c : integer;
begin
  if (ParamCount = 0) or (ParamStr(1) = '/?') then
  begin
    writeln('usage: iskeydown <key1> [<key2>...]');
    writeln('       iskeydown /list');
    writeln;
    writeln('iskeydown checks if ALL of the keys given on the command line');
    writeln('are currently pressed. it returns 0 if are all pressed,');
    writeln('1 if not all are down and 2 if an error occurred.');
    writeln;
    writeln('you can either use the VK_XYZ windows api key names or the');
    writeln('numerical scancode values');
    writeln;
    writeln('  e.g. "iskeydown VK_LEFT VK_SHIFT" returns 0 only if the');
    writeln('  left arrow key AND one of the shift keys are pressed.');
    writeln;
    writeln('iskeydown /list prints a list of all known keycodes to stdout.');
    writeln;
    writeln('(C) markus stephany, released under the terms of the GNU GPL.');
    writeln('iskeydown version: '+BT_VERSION);
    Halt(2);
  end
  else
    if ParamStr(1) = '/list' then
    begin
      for i := Low(KEY_NAMES) to High(KEY_NAMES) do
        writeln(KEY_NAMES[i].N,': ',KEY_NAMES[i].D);
    end
    else
    begin
      for i := 1 to ParamCount do
      begin
        c := GetKey(ParamStr(i));
        if c = -1 then
        begin
          writeln('iskeydown error: unknown key code "',ParamStr(i),'"');
          Halt(2);
        end;
        if (GetAsyncKeyState(c) and (not 1)) = 0 then
          Halt(1); // not currently down
      end;
      Halt(0); // all keys down
    end;
end.

