unit utils;

(*

  some utility functions for gpscript/gpcvreg

  unit version: 0.02, alpha stage!

  (C) markus stephany merkes@mirkes.de, released under the GNU GPL

*)

interface
uses
  Windows,
  SysUtils;

// convert ansi to oem
function ToOem(const S: string): string;

// trim and evtly unquote a string
function UnQuote(const SIn: string): string;

// data conversion
function GetDataTypeString(dwType: Cardinal): string;
function GetStringValue(iSize: Integer; pStr: PChar): string;
function GetBinaryValue(iSize: Integer; pData: PByte): string;
function GetDWORDValue(iSize: Integer; pData: PDWORD): string;

// find one of the cmdline switches
function FindACmdLineSwitch(const Switches: array of string): Boolean;

// get the system directory
function GetSysDir: string;

// crack a textual reg key name
procedure CrackRegKey(strKey: string; var HKRoot: HKEY; var strPath: string);

// pattern match (adopted to win32 from fnmatch/linux.pp/free pascal 1.0.6)
function PatternMatch(strPattern, strFile: string; const boolIgnoreCase: Boolean = True): Boolean;

// count the number of given chars in a string
function CharSetCount(seChars: TSysCharSet; const strToCount: string): Integer;

// expand environment variables contained in a string
function ExpandEnvironmentVariables(const strIn: string): string;

// get computer and user name
function ComputerName: string;
function UserName: string;

// extract a token from a string
function ExtractToken(var S: string; seDelims, seRunOver: TSysCharSet): string;

// get a one char answer from stdin
function Choice(strMsg, strKeys: string; chrDef: Char): Char;

// replace some meta chars with special values
function SubstituteSpecialVars(const strIn, strMsg: string): string;

resourcestring
  STR_ERROR = 'ERROR: %s';

const
  RegTypeNames: array[REG_NONE..REG_RESOURCE_REQUIREMENTS_LIST] of string = (
    'REG_NONE',
    'REG_SZ',
    'REG_EXPAND_SZ',
    'REG_BINARY',
    'REG_DWORD',
    'REG_DWORD_BIG_ENDIAN',
    'REG_LINK',
    'REG_MULTI_SZ',
    'REG_RESOURCE_LIST',
    'REG_FULL_RESOURCE_DESCRIPTOR',
    'REG_RESOURCE_REQUIREMENTS_LIST'
    );

  RegKeyNames: array[HKEY_CLASSES_ROOT..HKEY_DYN_DATA] of
  record
    LongName,
      ShortName: string;
  end = (
    (LongName: 'HKEY_CLASSES_ROOT'; ShortName: 'HKCR'),
    (LongName: 'HKEY_CURRENT_USER'; ShortName: 'HKCU'),
    (LongName: 'HKEY_LOCAL_MACHINE'; ShortName: 'HKLM'),
    (LongName: 'HKEY_USERS'; ShortName: 'HKU'),
    (LongName: 'HKEY_PERFORMANCE_DATA'; ShortName: 'HKPD'),
    (LongName: 'HKEY_CURRENT_CONFIG'; ShortName: 'HKCC'),
    (LongName: 'HKEY_DYN_DATA'; ShortName: 'HKDD')
    );

  CLEFT =
    '(C) markus stephany, released under the terms of the GNU GPL.';

implementation

resourcestring
  // errors
  STR_ERR_ROOT_KEY_NAME = 'invalid root key name %s';

// convert ansi to oem

function ToOem(const S: string): string;
begin
  Result := S;
  UniqueString(Result);
  CharToOem(PChar(Result), PChar(Result));
end;

// trim and evtly unquote a string

function UnQuote(const SIn: string): string;
begin
  Result := Trim(SIn);
  if ((Copy(Result, 1, 1) = '"') and (Copy(Result, Length(Result), 1) = '"')) or
    ((Copy(Result, 1, 1) = '''') and (Copy(Result, Length(Result), 1) = ''''))
      then
  begin
    Delete(Result, 1, 1);
    Delete(Result, Length(Result), 1);
  end;
end;

function GetDataTypeString(dwType: Cardinal): string;
begin
  if dwType in [REG_NONE..REG_RESOURCE_REQUIREMENTS_LIST] then
    Result := RegTypeNames[dwType]
  else
    Result := '0x' + IntToHex(dwType, 8);
end;

function GetStringValue(iSize: Integer; pStr: PChar): string;
var
  str1: string;
  intLoop: Integer;
begin
  SetString(str1, pStr, iSize);
  Result := 'S "';
  if Copy(str1, Length(str1), 1) = #0 then
    Delete(str1, Length(str1), 1);
  if str1 <> '' then
    for intLoop := 1 to Length(str1) do
      case str1[intLoop] of
        #0..#31: Result := Result + '\0x' + IntToHex(Byte(str1[intloop]), 2);
        '\', '"': Result := Result + '\' + str1[IntLoop];
      else
        Result := Result + str1[intLoop]
      end;
  Result := Result + '"';
end;

function GetBinaryValue(iSize: Integer; pData: PByte): string;
var
  intLoop,
    intRun: Integer;
begin
  Result := 'B ';
  intRun := 3;
  for intLoop := 0 to Pred(iSize) do
  begin
    inc(intRun);
    if intRun >= 24 then
    begin
      Result := Result + '\'#13#10;
      intRun := 1;
    end;
    Result := Result + IntToHex(pData^, 2) + ' ';
    pData := Pointer(LongInt(pData) + 1);
  end;
end;

function GetDWORDValue(iSize: Integer; pData: PDWORD): string;
begin
  Result := '';
  if iSize <> sizeof(DWORD) then
    Result := GetBinaryValue(iSize, Pointer(pData))
  else
    Result := 'D 0x' + IntToHex(pData^, 8);
end;

// find one of the cmdline switches

function FindACmdLineSwitch(const Switches: array of string): Boolean;
var
  intLoop: Integer;
begin
  Result := False;
  for intLoop := Low(Switches) to High(Switches) do
    if FindCmdLineSwitch(Switches[intLoop], ['-', '/'], True) then
    begin
      Result := True;
      Break;
    end;
end;

// get the system directory

function GetSysDir: string;
var
  szBuf: array[0..MAX_PATH] of Char;
begin
  SetString(Result, szBuf, GetSystemDirectory(szBuf, sizeof(szBuf)));
  Result := IncludeTrailingPathDelimiter(Trim(Result));
end;

// crack a textual reg key name

procedure CrackRegKey(strKey: string; var HKRoot: HKEY; var strPath: string);
var
  hkTemp: HKEY;
  intPos: Integer;
begin
  HKRoot := INVALID_HANDLE_VALUE;
  intPos := Pos('\', strKey);
  if intPos = 0 then
    strPath := ''
  else
  begin
    strPath := Copy(strKey, intPos + 1, MaxInt);
    strKey := Copy(strKey, 1, intPos - 1);
  end;
  for hkTemp := Low(RegKeyNames) to High(RegKeyNames) do
    if (AnsiCompareText(strKey, RegKeyNames[hkTemp].LongName) = 0) or
      (AnsiCompareText(strKey, RegKeyNames[hkTemp].ShortName) = 0) then
    begin
      HKRoot := hkTemp;
      Break;
    end;
  if HKRoot = INVALID_HANDLE_VALUE then
    raise Exception.CreateFmt(STR_ERR_ROOT_KEY_NAME, [strKey]);
end;

// pattern match (adopted to win32 from fnmatch/linux.pp/free pascal 1.0.6)

function PatternMatch(strPattern, strFile: string; const boolIgnoreCase: Boolean
  =
  True): Boolean;
var
  LenPat, LenName: longint;
  function DoFNMatch(i, j: longint): Boolean;
  var
    Found: boolean;
  begin
    Found := true;
    while Found and (i <= LenPat) do
    begin
      case strPattern[i] of
        '?': Found := (j <= LenName);
        '*':
          begin
            {find the next character in pattern, different of ? and *}
            while Found and (i < LenPat) do
            begin
              inc(i);
              case strPattern[i] of
                '*': ;
                '?':
                  begin
                    inc(j);
                    Found := (j <= LenName);
                  end;
              else
                Found := false;
              end;
            end;
            {Now, find in name the character which i points to, if the * or ?
             wasn't the last character in the pattern, else, use up all the
             chars in name}
            Found := true;
            if (i <= LenPat) then
            begin
              repeat
                {find a letter (not only first !) which maches pattern[i]}
                while (j <= LenName) and (strFile[j] <> strPattern[i]) do
                  inc(j);
                if (j < LenName) then
                begin
                  if DoFnMatch(i + 1, j + 1) then
                  begin
                    i := LenPat;
                    j := LenName; {we can stop}
                    Found := true;
                  end
                  else
                    inc(j); {We didn't find one, need to look further}
                end;
              until (j >= LenName);
            end
            else
              j := LenName; {we can stop}
          end;
      else {not a wildcard character in pattern}
        Found := (j <= LenName) and (strPattern[i] = strFile[j]);
      end;
      inc(i);
      inc(j);
    end;
    Result := Found and (j > LenName);
  end;
begin {start FNMatch}
  if boolIgnoreCase then
  begin
    strPattern := AnsiUpperCase(strPattern);
    strFile := AnsiUpperCase(strFile);
  end;
  LenPat := Length(strPattern);
  LenName := Length(strFile);
  Result := DoFNMatch(1, 1);
end;

// count the number of given chars in a string

function CharSetCount(seChars: TSysCharSet; const strToCount: string): Integer;
var
  intLoop: Integer;
begin
  Result := 0;
  if Bool(Length(strToCount)) then
    for intLoop := 1 to Length(strToCount) do
      if strToCount[intLoop] in seChars then
        Inc(Result);
end;

// expand environment variables contained in a string

function ExpandEnvironmentVariables(const strIn: string): string;
var
  intLen: Integer;
begin
  intLen := ExpandEnvironmentStrings(PChar(strIn), nil, 0);
  if intLen = 0 then
    Result := strIn
  else
  begin
    SetLength(Result, intLen + 1);
    SetLength(Result, ExpandEnvironmentStrings(PChar(strIn), PChar(Result),
      Length(Result)));
    while (Result <> '') and (Result[Length(Result)] = #0) do
      Delete(Result, Length(Result), 1);
    if Result = '' then
      Result := strIn;
  end;
end;

// get computer and user name

function ComputerName: string;
var
  szBuf: array[0..MAX_COMPUTERNAME_LENGTH] of Char;
  dwSize: Cardinal;
begin
  dwSize := sizeof(szBuf);
  if GetComputerName(szBuf, dwSize) then
    SetString(Result, szBuf, dwSize);
end;

function UserName: string;
var
  szBuf: array[0..MAX_PATH] of Char;
  dwSize: Cardinal;
begin
  dwSize := sizeof(szBuf);
  if GetUserName(szBuf, dwSize) then
    SetString(Result, szBuf, dwSize - 1);
end;

// extract a token from a string

function ExtractToken(var S: string; seDelims, seRunOver: TSysCharSet): string;
var
  boolInRun: Boolean;
begin
  Result := '';
  boolInRun := False;
  while S <> '' do
  begin
    if S[1] in seRunOver
    then
      boolInRun := not boolInRun;
    if (S[1] in seDelims) and (not boolInRun) then
    begin
      Delete(S,1,1);
      Break;
    end;
    Result := Result+S[1];
    Delete(S,1,1);
  end;
end;

// get a one char answer from stdin
function Choice(strMsg, strKeys: string; chrDef: Char): Char;
var
  chrGot: Char;
begin
  write(ToOem(Format('%s [%s] ',[strMsg, strKeys])));

  repeat
    Read(chrGot);
  until (Pos(UpCase(chrGot), UpperCase(strKeys)) > 0) or (chrGot in [#13,#10]);

  if not (chrGot in [#13,#10]) then
  begin
    Result := UpCase(chrGot);
    writeln;
  end
  else
    Result := UpCase(chrDef);
end;

// replace some meta chars with special values
function SubstituteSpecialVars(const strIn, strMsg: string): string;
var
  intLoop: Integer;
  strTemp: string;
begin
  Result := '';
  intLoop := 1;
  while intLoop <= Length(strIn) do
  begin
    if strIn[intLoop] = '$' then
    begin
      inc(intLoop);
      if intLoop <= Length(strIn) then
        case strIn[intLoop] of
          'M': Result := Result + strMsg;
          '_': Result := Result + sLineBreak;
          'A': Result := Result + '&';
          'B': Result := Result + '|';
          'C': Result := Result + '(';
          'D': Result := Result + DateToStr(Now);
          'E': Result := Result + '%';
          'F': Result := Result + ')';
          'G': Result := Result + '>';
          'H': Delete(Result, Length(Result), 1);
          'I': Result := Result + '''';
          'L': Result := Result + '<';
          'N': Result := Result + Copy(GetCurrentDir, 1, 2);
          'P': Result := Result + ExcludeTrailingPathDelimiter(GetCurrentDir);
          'Q': Result := Result + '=';
          'R':
            begin
              strTemp := ExtractFileDrive(ExpandUNCFileName(GetCurrentDir));
              if Copy(strTemp, 1, 2) = '\\' then
                Result := Result + ExcludeTrailingPathDelimiter(strTemp);
            end;
          'S': Result := Result + ' ';
          'T': Result := Result + TimeToStr(Now);
          'U': Result := Result + UserName;
          'W': Result := Result + ComputerName;
          'Y': Result := Result + '"';
          '$': Result := Result + '$';
        else
          Result := Result + strIn[intLoop];
        end;
    end
    else
      Result := Result + strIn[intLoop];
    inc(intLoop);
  end;
end;

end.

