program logger;

(*

  logger: an echo replacement that also logs to a lige file in a specific format

  program version: 0.01, alpha stage!

  (C) markus stephany merkes@mirkes.de, released under the GNU GPL, version 2

*)

{$APPTYPE CONSOLE}

uses
  Windows,
  SysUtils,
  utils;

{$I ..\LIB\VERSION.INC}

const
  LoggerVersion = 'logger '+BT_VERSION; // my own version

  // show help

procedure Usage;
begin
  writeln(ErrOutput, LoggerVersion);
  writeln(ErrOutput, CLEFT);
  writeln(ErrOutput);
  writeln(ErrOutput,
    'logger is an enhanced ECHO command that additionally logs to a given log file.');
  writeln(ErrOutput,
    'the output to the log file can be formatted in various formats. all options');
  writeln(ErrOutput,
    'and the log file name are read from the environment variable LOGGER_OPTS.');
  writeln(ErrOutput);
  writeln(ErrOutput, 'usage:');
  writeln(ErrOutput,
    'logger <message to log>');
  writeln(ErrOutput);
  writeln(ErrOutput);
  writeln(ErrOutput, '  the LOGGER_OPTS environment variable:');
  writeln(ErrOutput,
    '    the contents of this variable tell logger how to format the output to');
  writeln(ErrOutput,
    '    the lo file (the output to STDOUT is unmodified though), different options');
  writeln(ErrOutput,
    '    are separated by semicola ";".');
  writeln(ErrOutput);
  writeln(ErrOutput,
    '  options (from LOGGER_OPTS, unknown options will be ignored):');
  writeln(ErrOutput, '    LOGFILE=<filename>');
  writeln(ErrOutput,
    '      logger will write the message to this file. if no logfile name is');
  writeln(ErrOutput,
    '      specified, logger will write the output to a file called "logger.log"');
  writeln(ErrOutput,
    '      in the current directory. if this file name is set to "$", the behaviour');
  writeln(ErrOutput,
    '      of logger will change: it will not use a log file, but writes the');
  writeln(ErrOutput, '      formatted output to STDOUT.');
  writeln(ErrOutput);
  writeln(ErrOutput, '    NOOPTS=<0|1>');
  writeln(ErrOutput,
    '      if this option is set to 0 (default), a single argument "/?" will lead');
  writeln(ErrOutput,
    '      to a help message. if set to 1, "/?" will be written as is.');
  writeln(ErrOutput);
  writeln(ErrOutput, '    NOSTDOUT=<0|1>');
  writeln(ErrOutput,
    '      if this option is set to 1, output to STDOUT will be suppressed. if set');
  writeln(ErrOutput,
    '      to 0 (default), logger will additionally echo the message to STDOUT.');
  writeln(ErrOutput);
  writeln(ErrOutput, '    TRIM=<0|1>');
  writeln(ErrOutput,
    '      if this option is set to 1, leading and following whitespaces and');
  writeln(ErrOutput,
    '      quotes will be removed from the message. default is 0.');
  writeln(ErrOutput);
  writeln(ErrOutput, '    FORMAT=<format specifier>');
  writeln(ErrOutput,
    '      tells logger how to format the output to the log file. the default');
  writeln(ErrOutput, '      is "$M$_" (see below).');
  writeln(ErrOutput);
  writeln(ErrOutput, '  formatting output:');
  writeln(ErrOutput,
    '    logger understands the following special character combinations that');
  writeln(ErrOutput,
    '    will be replaced (much like the windows "prompt" command):');
  writeln(ErrOutput);
  writeln(ErrOutput, '      $M  the message itself');
  writeln(ErrOutput, '      $_  carriage return followed by a line feed');
  writeln(ErrOutput, '      $A  ampersand &');
  writeln(ErrOutput, '      $B  pipe |');
  writeln(ErrOutput, '      $C  left parenthesis (');
  writeln(ErrOutput, '      $D  current date');
  writeln(ErrOutput, '      $E  percent sign %');
  writeln(ErrOutput, '      $F  right parenthesis )');
  writeln(ErrOutput, '      $G  greater-than sign >');
  writeln(ErrOutput, '      $H  removes the preceding character');
  writeln(ErrOutput, '      $I  single quote ''');
  writeln(ErrOutput, '      $L  less-than sign <');
  writeln(ErrOutput, '      $N  current drive');
  writeln(ErrOutput, '      $P  current path');
  writeln(ErrOutput, '      $Q  equal sign =');
  writeln(ErrOutput, '      $R  name of the remote share of the current drive (if any)');
  writeln(ErrOutput, '      $S  blank (space)');
  writeln(ErrOutput, '      $T  current time');
  writeln(ErrOutput, '      $U  current user name');
  writeln(ErrOutput, '      $W  (windows) computer name');
  writeln(ErrOutput, '      $Y  double quote "');
  writeln(ErrOutput, '      $$  dollar sign $');
  writeln(ErrOutput);
  writeln(ErrOutput, '    all other characters will be written literally.');
  Halt(1);
end;

resourcestring
  // error hints
  STR_ERR_NO_VALUE = 'No value was given for option %s';
  STR_ERR_INVALID_VALUE01 = 'Invalid value %s for option %s (must be 0 or 1)';
  STR_ERR_UNKNOWN_OPT = 'Unknown option: %s';

var
  BoolNoStdOut: Boolean = False; // no echo
  BoolNoOpts: Boolean = False; // dont recognize /? option
  BoolTrim: Boolean = False; // remove leading ws and quotes from message
  StrLogFile: string = 'logger.log'; // log file
  StrFormat: string = '$M$_'; // message format
  StrMessage: string; // actual message from argv

const
  // my options
  MY_COMMANDS: array[0..4] of string = (
    'NOSTDOUT',
    'LOGFILE',
    'FORMAT',
    'NOOPTS',
    'TRIM'
    );

  MY_VARNAME = '%LOGGER_OPTS%'; // my environment variable

  // check trimming of leading and following garbage

function CheckTrim(strIn: string): string;
begin
  Result := strIn;
  if BoolTrim then
  begin
    while (Result <> '') and (Result[1] in [#0..#32, '''', '"']) do
      Delete(Result, 1, 1);

    while (Result <> '') and (Result[Length(Result)] in [#0..#32, '''', '"']) do
      Delete(Result, Length(Result), 1);
  end;
end;

// check 0/1 value

procedure CheckBoolNumInput(intCmd: integer; strVal: string; var intVal:
  integer);
begin
  if Trim(strVal) = '0' then
    intVal := 0
  else if Trim(strVal) = '1' then
    intVal := 1
  else
    raise Exception.CreateFmt(STR_ERR_INVALID_VALUE01, [strVal,
      MY_COMMANDS[intCmd]]);
end;

var
  strOutput,
    strOpt,
    strVal,
    strOptions: string; // all options from environment

  intCmd,
    intPos: integer;

  F: System.Text;
begin
  try
    // get message to print from cmdline
    StrMessage := GetCommandLine;
    ExtractToken(StrMessage, [' '], ['"']); // skip over program name

    // get options
    strOptions := UnQuote(ExpandEnvironmentVariables(MY_VARNAME));
    if strOptions = MY_VARNAME then
      strOptions := '';

    // parse options
    while strOptions <> '' do
    begin
      strOpt := ExtractToken(strOptions, [';'], ['"', '''']);
      strOpt := UnQuote(strOpt);
      if strOpt <> '' then
      begin
        intPos := Pos('=', strOpt);
        if intPos = 0 then
          raise Exception.CreateFmt(STR_ERR_NO_VALUE, [strOpt]);
        strVal := Copy(strOpt, intPos + 1, MaxInt);
        strOpt := Copy(strOpt, 1, intPos - 1);

        intCmd := -1;
        for intPos := Low(MY_COMMANDS) to High(MY_COMMANDS) do
          if AnsiCompareText(MY_COMMANDS[intPos], strOpt) = 0 then
          begin
            intCmd := intPos - Low(MY_COMMANDS);
            Break;
          end;

        case intCmd of
          0, 3, 4:
            begin
              CheckBoolNumInput(intCmd, strVal, intPos);
              case intCmd of
                0: BoolNoStdOut := intPos = 1;
                3: BoolNoOpts := intPos = 1;
                4: BoolTrim := intPos = 1;
              end;
            end;
          1: StrLogFile := UnQuote(strVal);
          2: StrFormat := UnQuote(strVal);
        else
          raise Exception.CreateFmt(STR_ERR_UNKNOWN_OPT, [strOpt]);
        end;
      end;
    end;

    // check if help wanted and not disabled
    if (Trim(StrMessage) = '/?') and (not BoolNoOpts) then
      Usage;

    // build output
    strOutput := SubstituteSpecialVars(StrFormat, CheckTrim(StrMessage));

    // output
    if StrLogFile = '$' then
      write(strOutput)
    else
    begin
      if not BoolNoStdOut then
        writeln(strMessage);
      Assign(F, StrLogFile);
      if not FileExists(StrLogFile) then
        Rewrite(F)
      else
        Append(F);
      try
        write(F, strOutput);
      finally
        Close(F);
      end;
    end;

  except
    on E: Exception do
    begin
      WriteLn(ErrOutput, ToOem(Format(STR_ERROR, [E.Message])));
      if ExitCode = 0 then
        ExitCode := 1;
    end;
  end;
end.

